﻿using DataV.Enums;
using DataV.Model;
using DataV.Service.Dto;
using Furion.DependencyInjection;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Xml;
using XExten.Advance.HttpFramework.MultiFactory;

namespace DataV.Service
{
    public class DataVService : IDataVService, ITransient
    {
        private SqlDbContext Db;

        public DataVService()
        {
            Db = new SqlDbContext();
        }
        /// <summary>
        /// 大屏数据
        /// </summary>
        /// <param name="autokey"></param>
        /// <returns></returns>
        public Result<object> GetAllOrderDataState(int? autokey)
        {
            var db = Db.DbContext();

            List<string> OrderIds = new List<string>();


            if (autokey != null)
            {
                OrderIds = db.Queryable<BPA_StoreInfo>().LeftJoin<BPA_SubOrder>((t, x) => t.Id == x.StoreId)
                    .Where(t => t.AutoKey == autokey.Value).Select((t, x) => x.OrderId).ToList();
            }

            var data = db.Queryable<BPA_Order>().Where(t => t.AfterStatus == 0)
                   .Where(t => t.OrderStatus == PayEnum.PayOk || t.PayMode == PayModelEnum.Other)
                   .Where(t => !string.IsNullOrEmpty(t.TransactionId))
                   .Where(t => !string.IsNullOrEmpty(t.TradeNo))
                   .WhereIF(OrderIds.Count > 0, t => OrderIds.Contains(t.Id))
                  .Select(t => new { t.OrderRealMoney, t.CreatedAt }).ToList();

            return new Result<object>
            {
                Data = new
                {
                    Total = data.Sum(t => t.OrderRealMoney).ToString(),
                    OrderCount = data.Count().ToString(),
                    MoneyM = data.Where(t => t.CreatedAt.Value.Month == DateTime.Now.Month).Sum(t => t.OrderRealMoney).ToString(),
                    MoneyD = data.Where(t => t.CreatedAt.Value >= DateTime.Parse(DateTime.Now.ToShortDateString()) && t.CreatedAt < DateTime.Parse(DateTime.Now.ToShortDateString()).AddDays(1)).Sum(t => t.OrderRealMoney).ToString(),
                },
                IsSuccess = true,
                Msg = String.Empty
            };
        }

        /// <summary>
        /// 设备
        /// </summary>
        /// <returns></returns>
        public Result<Dictionary<string, List<int>>> OnlineDevice()
        {
            string sql = $@"SELECT
	a.AutoKey,
	b.Name 
FROM
	BPA_StoreInfo AS a
	LEFT JOIN BPA_Company AS b ON a.GroupId= b.Id
";

            var data = Db.DbContext().SqlQueryable<DataVDto>(sql).ToList().GroupBy(t => t.Name).Select(t => new
            {
                Key = t.Key,
                Value = t.Select(x => x.AutoKey).ToList()
            }).ToDictionary(t => t.Key, t => t.Value);


            return new Result<Dictionary<string, List<int>>>
            {
                Data = data,
                IsSuccess = true,
                Msg = String.Empty
            };
        }

        /// <summary>
        /// 折线图
        /// </summary>
        /// <param name="autokey"></param>
        /// <returns></returns>
        public Result<object> OrderLine(int? autokey)
        {
            var db = Db.DbContext();

            List<string> OrderIds = new List<string>();


            if (autokey != null)
            {
                OrderIds = db.Queryable<BPA_StoreInfo>().LeftJoin<BPA_SubOrder>((t, x) => t.Id == x.StoreId)
                    .Where(t => t.AutoKey == autokey.Value).Select((t, x) => x.OrderId).ToList();
            }

            var data = db.Queryable<BPA_Order>()
                  .Where(t => !string.IsNullOrEmpty(t.TransactionId))
                  .WhereIF(OrderIds.Count > 0, t => OrderIds.Contains(t.Id))
                  .Where(t => !string.IsNullOrEmpty(t.TradeNo)).Select(t => new
                  {
                      t.OrderStatus,
                      t.AfterStatus,
                      t.CreatedAt,
                      t.PayMode
                  }).ToList();

            List<object> res = new List<object>();


            for (int index = 0; index < 10; index++)
            {
                var temp = DateTime.Now.AddDays(-index);
                var end = DateTime.Parse(temp.ToString("yyyy-MM-dd 23:59:59"));
                var start = DateTime.Parse(temp.ToString("yyyy-MM-dd 00:00:00"));
                var NO = data.Where(t => t.CreatedAt.Value < end && t.CreatedAt.Value >= start).Where(t => t.AfterStatus != 0 && (t.OrderStatus == PayEnum.PayOk || t.PayMode == PayModelEnum.Other)).Count();
                var OK = data.Where(t => t.CreatedAt.Value < end && t.CreatedAt.Value >= start).Where(t => t.AfterStatus == 0 && (t.OrderStatus == PayEnum.PayOk || t.PayMode == PayModelEnum.Other)).Count();
                var Date = temp.ToString("MM/dd");
                res.Add(new
                {
                    NO,
                    OK,
                    Date
                });
            }

            return new Result<object>
            {
                Data = res,
                IsSuccess = true,
                Msg = String.Empty,
            };
        }

        /// <summary>
        /// 下单数量
        /// </summary>
        /// <param name="autokey"></param>
        /// <returns></returns>
        public Result<object> OrderNumber(int? autokey)
        {
            var db = Db.DbContext();

            List<string> OrderIds = new List<string>();


            if (autokey != null)
            {
                OrderIds = db.Queryable<BPA_StoreInfo>().LeftJoin<BPA_SubOrder>((t, x) => t.Id == x.StoreId)
                    .Where(t => t.AutoKey == autokey.Value).Select((t, x) => x.OrderId).ToList();
            }


            var data = db.Queryable<BPA_Order>().Where(t => t.AfterStatus == 0)
                    .Where(t => t.OrderStatus == PayEnum.PayOk || t.PayMode == PayModelEnum.Other)
                    .Where(t => !string.IsNullOrEmpty(t.TransactionId))
                    .Where(t => !string.IsNullOrEmpty(t.TradeNo))
                    .WhereIF(OrderIds.Count > 0, t => OrderIds.Contains(t.Id))
                   .Select(t => t.CreatedAt).ToList();

            return new Result<object>
            {
                IsSuccess = true,
                Msg = String.Empty,
                Data = new
                {
                    Year = data.Where(t => t.Value.Year == DateTime.Now.Year).Count(),
                    Month = data.Where(t => t.Value.Month == DateTime.Now.Month).Count(),
                    Day = data.Where(t => t.Value.Day == DateTime.Now.Day).Count(),
                }
            };
        }

        /// <summary>
        /// 区域销售
        /// </summary>
        /// <returns></returns>
        public Result<object> GetLocSale()
        {
            var db = Db.DbContext();

            var baiduMap = "http://api.map.baidu.com/geocoder/v2/?ak=3ce7ddfda60046708248ea55e10b1435&callback=renderReverse&location={0}&output=xml";
            List<LocDto> loc = new List<LocDto>();
            var data = db.SqlQueryable<dynamic>(@"SELECT
	a.Store_Loc AS Loc,
	(
	SELECT COUNT
		( b.Id ) 
	FROM
		BPA_Order AS b
		JOIN BPA_SubOrder AS c ON b.Id= c.OrderId 
	WHERE
		c.StoreId= a.Id 
		AND ( b.OrderStatus= 1 OR b.OrderStatus = 4 ) 
		AND b.TradeNo IS NOT NULL 
		AND b.TransactionId IS NOT NULL 
	) AS Counts 
FROM
	BPA_StoreInfo AS a 
WHERE
	a.Store_Status = 0 
	AND a.IsDeleted = 0 
	AND a.Store_Loc LIKE '%,%'").ToList();
            IHttpMultiClient http = null;

            data.ForEach(item =>
            {
                if (http == null)
                    http = IHttpMultiClient.HttpMulti.AddNode(opt =>
                    {
                        opt.NodePath = String.Format(baiduMap, item.Loc);
                    });
                else
                    http = http.AddNode(opt =>
                    {
                        opt.NodePath = String.Format(baiduMap, item.Loc);
                    });
            });
            XmlDocument doc = new XmlDocument();

            var Temps = http.Build().RunString();

            for (int i = 0; i < Temps.Count; i++)
            {
                doc.LoadXml(Temps[i]);
                loc.Add(new LocDto
                {
                    Count = data[i].Counts,
                    City = doc.SelectSingleNode("//province")?.InnerText + doc.SelectSingleNode("//city")?.InnerText
                });
            }

            return new Result<object>
            {
                Data = new
                {
                    Data = loc.GroupBy(t => t.City).Select(t => new LocDto
                    {
                        City = t.Key,
                        Count = t.Sum(t => t.Count)
                    })
                },
                IsSuccess = true,
                Msg = ""
            };
        }
    }
}
